/* LINKER SCRIPT FOR ADuC7026 */

OUTPUT_ARCH(arm)
ENTRY(_start)

/*-------------------------------*/
/*DEFINITIONS*/
/*-------------------------------*/

/*Specify available Memory*/
	MEMORY {
		flash (rx) : ORIGIN = 0x00080000, LENGTH = 64K
		ram (rwx) : ORIGIN = 0x00010000, LENGTH = 8K
	}


/*Specify Stack Size*/
	_USER_STACK_SIZE_	= 512;
	_FIQ_STACK_SIZE_	= 128;
	_IRQ_STACK_SIZE_	= 128;
	_SUP_STACK_SIZE_	= 8;
	_ABORT_STACK_SIZE_	= 8;
	_UNDEF_STACK_SIZE_	= 8;

/*-------------------------------*/
/*SECTIONS*/
/*-------------------------------*/

SECTIONS {
	. = 0x00080000;
	
/*--------------*/
/*.TEXT*/
/*--------------*/

	/*Merge .text Sections at Beginning of FLASH*/
	.text : {
			* (exception_vectors);	/* Primary (dummy) Vectors first, so reset-address is at 0x0 */
			* (.text .text.*);				/*  "*" is wildcard for .text sections of all .o-files */
			* (.rodata);
			* (.rodata*);
			* (.glue_7);
			* (.glue_7t);
			. = ALIGN(0x4);
	} >flash
	. = ALIGN(0x4);
	_TEXT_END_ = .;

	
/*-------------*/
/*RAM VECTORS */
/*-------------*/
	/*if memory remap is performed, space needs to be reserved at beginning of ram for vector table */
			. = 0x00010000;					/* . moved to beginning of RAM */
			_RAM_START_ = . ;
	.ramvect : {
			. += 0x00000020;				/* reserve enough space for Vector Table */
			. = ALIGN(0x4);
	} >ram
	. = ALIGN(0x4);
	
/*--------------*/
/*.FASTCODE*/
/*--------------*/
	/* .fastcode Sections go after Vector Table in RAM */
	_FLASH_FCODE_START_ = ADDR (.fastcode);
	_RAM_FCODE_START_ = . ;
	
	.fastcode : AT (_FLASH_FCODE_START_) {	
		* (.text.fastcode);
		. = ALIGN(0x4);
	} >ram
	. = ALIGN(0x4);
	_FCODE_LENGTH_ = . - _RAM_FCODE_START_;
	

/*--------------*/
/*.Data*/
/*--------------*/


	/*Merge Data Sections: First in FLASH, but linked in RAM (runtime address) */
	_FLASH_DATA_START_ = ADDR(.data);
	_RAM_DATA_START_ = .;
	
	.data : AT (_FLASH_DATA_START_){	/* Data linked to runtime-address in RAM. loadtime-address specified by "AT()"*/
		* (.data);						/* to end of .text section in Flash. Copying done in assembler startup code */
		. = ALIGN(0x4);
	} >ram
	
	. = ALIGN(0x4);
	_DATA_LENGTH_ = . - _RAM_DATA_START_;
	
/*-------------*/
/* .BSS*/
/*-------------*/

	/*Merge uninitialised Data directly in RAM*/
	_BSS_START_ = .;
	.bss : {
		* (.bss);
		. = ALIGN(0x4);
	} >ram
	. = ALIGN(0x4);				/* Align to make sure that .bss section is setup correctly*/
	_BSS_END_ = .;


/*--------------*/
/* STACK*/
/*--------------*/

	/*Reserve room for Stack*/
	.stack : {
		_STACK_START_ = .;
		. += _FIQ_STACK_SIZE_ ;
		. = ALIGN(0x4);
		_FIQ_STACK_TOP_ = . ;
		. += _IRQ_STACK_SIZE_ ;
		. = ALIGN(0x4);
		_IRQ_STACK_TOP_ = .;
		. += _SUP_STACK_SIZE_ ;
		. = ALIGN(0x4);
		_SUP_STACK_TOP_ = .;
		. += _ABORT_STACK_SIZE_ ;
		. = ALIGN(0x4);
		_ABORT_STACK_TOP_ = .;
		. += _UNDEF_STACK_SIZE_ ;
		. = ALIGN(0x4);
		_UNDEF_STACK_TOP_ = .;
		. += _USER_STACK_SIZE_ ;
		. = ALIGN(0x4);
		_USER_STACK_TOP_ = .;
		_STACK_TOP_ = .;
	} >ram
	
	_end = .;
	__end = _end;
	PROVIDE(end = .);
}
